package com.mcwl.pay.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.redis.RedisCache;
import com.mcwl.common.domain.IdsParam;
import com.mcwl.common.utils.DateUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.memberCenter.domain.RechargeRecord;
import com.mcwl.memberCenter.service.MemberService;
import com.mcwl.memberCenter.service.RechargeRecordService;
import com.mcwl.myInvitation.domain.Commission;
import com.mcwl.myInvitation.domain.Consume;
import com.mcwl.myInvitation.service.CommissionService;
import com.mcwl.myInvitation.service.ConsumeService;
import com.mcwl.pay.domain.OrderTrade;
import com.mcwl.pay.mapper.OrderTradeMapper;
import com.mcwl.pay.service.OrderTradeService;
import com.mcwl.resource.domain.enums.OrderTypeEnum;
import com.mcwl.system.service.ISysUserService;
import com.mcwl.system.service.impl.SysUserServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.pay.service.impl
 * @Filename：OrderTradeServiceImpl
 * @Description TODO
 * @Date：2025/1/3 14:51
 */
@Service
public class OrderTradeServiceImpl extends ServiceImpl<OrderTradeMapper, OrderTrade> implements OrderTradeService {

    @Autowired
    private OrderTradeMapper orderTradeMapper;
   @Autowired
   private SysUserServiceImpl userExtensionService;

   @Autowired
   private OrderTradeService orderTradeService;

    @Autowired
    private CommissionService commissionService;

    @Autowired
    private ConsumeService consumeService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private MemberService memberService;

    @Autowired
    private RechargeRecordService rechargeRecordService;

    @Autowired
    private RedisCache redisCache;


    @Override
    public List<OrderTrade> selectMallProductList(OrderTrade orderTrade) {
        // 创建一个 LambdaQueryWrapper 实例
        LambdaQueryWrapper<OrderTrade> queryWrapper = new LambdaQueryWrapper<>();

        // 使用非空检查来添加查询条件
        if (StringUtils.isNotNull(orderTrade.getCode())) {
            queryWrapper.eq(OrderTrade::getCode, orderTrade.getCode());
        }
        if (StringUtils.isNotNull(orderTrade.getUserId())) {
            queryWrapper.eq(OrderTrade::getUserId, orderTrade.getUserId());
        }
        if (StringUtils.isNotNull(orderTrade.getProductId())) {
            queryWrapper.eq(OrderTrade::getProductId, orderTrade.getProductId());
        }
        if (StringUtils.isNotNull(orderTrade.getProductName())) {
            queryWrapper.like(OrderTrade::getProductName, orderTrade.getProductName());
        }
        if (StringUtils.isNotNull(orderTrade.getUserName())) {
            queryWrapper.eq(OrderTrade::getUserName, orderTrade.getUserName());
        }
        if (StringUtils.isNotNull(orderTrade.getOrderTime())) {
            queryWrapper.eq(OrderTrade::getOrderTime, orderTrade.getOrderTime());
        }
        if (StringUtils.isNotNull(orderTrade.getOrderStatus())) {
            queryWrapper.eq(OrderTrade::getOrderStatus, orderTrade.getOrderStatus());
        }
        if (StringUtils.isNotNull(orderTrade.getPayStatus())) {
            queryWrapper.eq(OrderTrade::getPayStatus, orderTrade.getPayStatus());
        }
        if (StringUtils.isNotNull(orderTrade.getTotalAmount())) {
            queryWrapper.eq(OrderTrade::getTotalAmount, orderTrade.getTotalAmount());
        }
        if (StringUtils.isNotNull(orderTrade.getPaymentAmount())) {
            queryWrapper.eq(OrderTrade::getPaymentAmount, orderTrade.getPaymentAmount());
        }

        return orderTradeMapper.selectList(queryWrapper);
    }


    @Override
    public int insertMallProduct(OrderTrade orderTrade) {
     // 保存订单
    return orderTradeMapper.insert(orderTrade);
    }

    @Override
    public void deleteMallProductByIds(IdsParam ids) {
        orderTradeMapper.deleteBatchIds(ids.getIds());
    }

    @Override
    public int updateMallProduct(OrderTrade orderTrade) {
        return orderTradeMapper.updateById(orderTrade);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void orderHandler(OrderTrade orderTrade, String suffix, Map<String, String> params) {

        // 根据订单类型进行处理
        if (OrderTypeEnum.POINTS.getName().equals(suffix)) {
            pointsHandler(orderTrade, params);
        } else if (OrderTypeEnum.MEMBER.getName().equals(suffix)) {
            Long promotionId = redisCache.getCacheObject(params.get("out_trade_no") + "promotionId");
            memberHandler(orderTrade, promotionId);
        } else if (OrderTypeEnum.WALLET.getName().equals(suffix)) {
            walletHandler(orderTrade, params);
        } else {
            throw new RuntimeException("订单类型错误");
        }
        // 处理订单
        this.orderHandler(orderTrade, params);
        // 删除redis缓存
        redisCache.deleteObject(orderTrade.getCode());
        redisCache.deleteObject(params.get("out_trade_no") + "promotionId");

    }

    private void pointsHandler(OrderTrade orderTrade, Map<String, String> params) {

        // 获取支付金额
        String amount = params.get("buyer_pay_amount");
        Double points = Double.parseDouble(amount) * 100;
        Long userId = orderTrade.getUserId();
        SysUser user = sysUserService.selectUserById(userId);
        user.setFreePoints(user.getFreePoints() + points);
        sysUserService.updateUser(user);

        // 添加充值记录
        RechargeRecord rechargeRecord = new RechargeRecord();
        rechargeRecord.setUserId(userId);
        rechargeRecord.setCode(orderTrade.getCode());
        rechargeRecord.setAmount(Double.parseDouble(amount));
        rechargeRecord.setPoints(points);
        rechargeRecord.setCreateBy(user.getUserName());
        rechargeRecord.setUpdateBy(user.getUserName());
        rechargeRecord.setUpdateTime(new Date());
        rechargeRecordService.save(rechargeRecord);


    }

    private void walletHandler(OrderTrade orderTrade, Map<String, String> params) {

        // 获取支付金额
        String amount = params.get("buyer_pay_amount");

        BigDecimal amountBigDecimal = new BigDecimal(amount);

        Long userId = orderTrade.getUserId();
        SysUser user = sysUserService.selectUserById(userId);

        BigDecimal wallet = new BigDecimal(user.getWallet().toString());

        BigDecimal walletDecimal = wallet.add(amountBigDecimal);

        user.setWallet(walletDecimal.doubleValue());
        sysUserService.updateUser(user);

        // 添加充值记录
//        RechargeRecord rechargeRecord = new RechargeRecord();
//        rechargeRecord.setUserId(userId);
//        rechargeRecord.setCode(orderTrade.getCode());
//        rechargeRecord.setAmount(Double.parseDouble(amount));
//        rechargeRecord.setPoints(points);
//        rechargeRecord.setCreateBy(user.getUserName());
//        rechargeRecord.setUpdateBy(user.getUserName());
//        rechargeRecord.setUpdateTime(new Date());
//        rechargeRecordService.save(rechargeRecord);


    }

    private void memberHandler(OrderTrade orderTrade, Long promotionId) {
        Long userId = orderTrade.getUserId();
        Integer memberLevelId = orderTrade.getProductId();
        String paymentMethod = "aliPay";

        memberService.createUserMember(userId, memberLevelId.longValue(), paymentMethod, promotionId);


    }

    private void productHandler(OrderTrade orderTrade) {
        Long userId = orderTrade.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);
        double amount = 0.0;
        // 通过sysUser获取邀请人id
        Long inviterUserId = sysUser.getInviterUserId();
        if (Objects.nonNull(inviterUserId)) {
            amount = 0.5; // 固定0.5元提成
        }


        // 保存消费表
        Consume consume = new Consume();
        consume.setUserId(userId);
        consume.setAmount(Double.valueOf(orderTrade.getPaymentAmount()));
        consume.setConsumeDate(orderTrade.getOrderTime());
        consume.setCreateBy(orderTrade.getUserName());
        consume.setUpdateBy(orderTrade.getUserName());
        consume.setCreateTime(new Date());
        consume.setUpdateTime(new Date());
        consume.setRemark("购买商品");
        consumeService.save(consume);


        // 保存提成表
        Commission commission = new Commission();
        commission.setConsumeId(inviterUserId);
        commission.setAmount(amount);
        commission.setPayStatus(0);
        commission.setCreateBy(sysUser.getUserName());
        commission.setUpdateBy(sysUser.getUserName());
        commission.setCreateTime(new Date());
        commission.setUpdateTime(new Date());
        commission.setRemark("提成");
        commissionService.save(commission);

    }

    private void orderHandler(OrderTrade orderTrade, Map<String, String> params) {
        // 更新订单状态
        // 支付宝交易凭证号
        orderTrade.setPaymentMethod(params.get("trade_no"));
        orderTrade.setTransactionId(1);
        orderTrade.setOrderTime(DateUtils.parseDate(params.get("gmt_payment")));
        orderTrade.setOrderStatus(2);
        orderTrade.setPayStatus(2);
        String totalAmountStr = params.get("total_amount");
        if (totalAmountStr != null && !totalAmountStr.isEmpty()) {
            BigDecimal totalAmount = new BigDecimal(totalAmountStr);
            orderTrade.setTotalAmount(totalAmount.doubleValue());
        }
        String buyerPayAmountStr = params.get("buyer_pay_amount");
        if (buyerPayAmountStr != null && !buyerPayAmountStr.isEmpty()) {
            BigDecimal buyerPayAmount = new BigDecimal(buyerPayAmountStr);
            orderTrade.setPaymentAmount(buyerPayAmount.intValue());
        }
        orderTrade.setUpdateTime(new Date());
        orderTradeService.updateById(orderTrade);
    }
}
