package com.mcwl.resource.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.R;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.SysUserAttention;
import com.mcwl.resource.domain.SysUserInfo;
import com.mcwl.resource.domain.vo.PageVo;
import com.mcwl.resource.mapper.MallProductLikeMapper;
import com.mcwl.resource.mapper.ModelImageMapper;
import com.mcwl.resource.mapper.ModelMapper;
import com.mcwl.resource.mapper.SysUserAttentionMapper;
import com.mcwl.resource.service.SysUserAttentionService;
import com.mcwl.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 关注表 业务实现层
 *
 * @author DaiZibo
 * @date 2025/1/3
 * @apiNote
 */

@Service
public class SysUserAttentionServiceImpl implements SysUserAttentionService {


    @Autowired
    private ModelMapper modelMapper;

    @Autowired
    private MallProductLikeMapper mallProductLikeMapper;

    @Autowired
    private SysUserAttentionMapper sysUserAttentionMapper;

    @Autowired
    private ModelImageMapper modelImageMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Override
    public R<Boolean> addAttention(Long userId) {

        //查看是否已关注
        Boolean aBoolean = selectAttention(userId);
        if (aBoolean == true) {
            //取关
            sysUserAttentionMapper.deleteByUserId(SecurityUtils.getUserId(), userId);
            return R.ok(false);
        }

        //关注
        SysUserAttention sysUserAttention = SysUserAttention.builder().userId(SecurityUtils.getUserId())
                .toUserId(userId)
                .createName(SecurityUtils.getUsername())
                .createTime(new Date())
                .build();
        sysUserAttentionMapper.insert(sysUserAttention);
        return R.ok(true);
    }

    @Override
    public Boolean selectAttention(Long userId) {

        SysUserAttention sysUserAttention = sysUserAttentionMapper.selectAttention(SecurityUtils.getUserId(), userId);

        if (sysUserAttention == null) {
            return false;
        }

        return true;
    }

    @Override
    public SysUserInfo selectUserInfo() {

        Long userId = SecurityUtils.getUserId();
        // 粉丝数
        Long userBeanNum = sysUserAttentionMapper.selectBean(userId);

        // 关注数
        Long userAttentionNum = sysUserAttentionMapper.selectAttentionCount(userId);

        // 模型下载数
        Long modelDownloadNum = modelMapper.sumNumber(userId);

        // 模型运行次数
        Long modelRunNum = modelMapper.sumRunNumber(userId);

        // 模型点赞次数
        Long modelLikeNum = modelMapper.sumLikeNumber(userId);

        // 图片点赞次数
        Long imageLikeNum = modelImageMapper.sumLikeNumber(userId);

        return SysUserInfo.builder().bean(userBeanNum)
                .attention(userAttentionNum)
                .modelDownloadNum(modelDownloadNum)
                .modelRunNum(modelRunNum)
                .modelLikeNum(modelLikeNum)
                .imageLikeNum(imageLikeNum)
                .build();
    }

    @Override
    public PageInfo<SysUser> selectAttentionPage(PageVo pageVo) {

        PageHelper pageHelper = new PageHelper();
        pageHelper.startPage(pageVo.getPageNumber(),pageVo.getPageSize());

        //条件构造器
        List<SysUser> sysUserList = sysUserMapper.selectUserPage(SecurityUtils.getUserId());


        return new PageInfo<SysUser>(sysUserList);
    }

    @Override
    public PageInfo<SysUser> selectToAttentionPage(PageVo pageVo) {

        PageHelper pageHelper = new PageHelper();
        pageHelper.startPage(pageVo.getPageNumber(),pageVo.getPageSize());

        //条件构造器
        List<SysUser> sysUserList = sysUserMapper.selectToUserPage(SecurityUtils.getUserId());


        return new PageInfo<SysUser>(sysUserList);
    }
}
